
package net.sf.freecol.common.util;


import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.sf.freecol.client.gui.i18n.Messages;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


/**
 * A class that makes it slightly tidier to create, parse and format XML
 * documents.
 */
public final class Xml
{
    public static final  String  COPYRIGHT = "Copyright (C) 2003-2006 The FreeCol Team";
    public static final  String  LICENSE = "http://www.gnu.org/licenses/gpl.html";
    public static final  String  REVISION = "$Revision: 1.1 $";


    // ------------------------------------------------------ class API methods

    public static Document newDocument() {

        try {
            DocumentBuilderFactory  factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder  builder = factory.newDocumentBuilder();
            return builder.newDocument();
        }
        catch ( ParserConfigurationException e ) {
            throw new Exception( e );
        }
    }


    public static Document documentFrom( String string ) {

        return documentFrom( new InputSource(new StringReader(string)) );
    }


    public static Document documentFrom( InputStream stream ) {

        return documentFrom( new InputSource(stream) );
    }


    public static String toString( Document document ) {

        return document.getDocumentElement().toString();
    }


    public static boolean hasAttribute( Node xmlElement, String attributeName ) {

        return xmlElement.getAttributes().getNamedItem(attributeName) != null;
    }


    public static String attribute( Node xmlElement, String attributeName ) {

        return xmlElement.getAttributes().getNamedItem(attributeName).getNodeValue();
    }


    public static String messageAttribute( Node xmlElement, String attributeName ) {

        return Messages.message( attribute(xmlElement, attributeName) );
    }


    public static int intAttribute( Node xmlElement, String attributeName ) {

        return Integer.parseInt( attribute(xmlElement, attributeName) );
    }


    public static boolean booleanAttribute( Node xmlElement, String attributeName ) {

        return parseTruth( attribute(xmlElement, attributeName) );
    }


    public static void forEachChild( Node xml, Method method ) {

        NodeList  childList = xml.getChildNodes();

        for ( int ci = 0, nc = childList.getLength();  ci < nc;  ci ++ ) {
            Node  child = childList.item( ci );

            if ( child instanceof Element ) {

                method.invokeOn( child );
            }
        }
    }


    // -------------------------------------------------- class support methods

    private static Document documentFrom( InputSource source ) {

        DocumentBuilderFactory  factory = DocumentBuilderFactory.newInstance();

        try {
            DocumentBuilder  builder = factory.newDocumentBuilder();
            return builder.parse( source );
        }
        catch ( ParserConfigurationException e ) {
            throw new Exception( e );
        }
        catch ( SAXException e ) {
            throw new Exception( e );
        }
        catch ( IOException e ) {
            throw new Exception( e );
        }
    }


    private static boolean parseTruth( String truthAsString )
    {
        if ( "yes".equals(truthAsString) ) {
            return true;
        }
        else if ( "no".equals(truthAsString) ) {
            return false;
        }
        throw new RuntimeException( "mus be 'yes' or 'no': " + truthAsString );
    }


    // ----------------------------------------------------------- constructors

    private Xml() {
    }


    // ----------------------------------------------------------- nested types

    public interface Method {

        public void invokeOn( Node xml );
    }


    /**
     * This class is defined so that exceptions thrown by methods on {@link Xml}
     * may be filtered from other runtime exceptions such as
     * {@link NullPointerException} if desired.  In the majority of cases, an
     * exception will mean that the game cannot proceed, which is why this is a
     * runtime exception so that it percolates up to the {@link Thread#run()}
     * method of the invoking thread or other controlling loop without the need
     * for every other method on the way to declare "throws" clauses.
     */
    public static final class Exception extends RuntimeException {

        Exception( Throwable cause ) {

            super( cause );
        }
    }

}
